import * as React from 'react';
import { getThemedSettings } from '@uifabricshared/themed-settings';
import { getSettings, returnAsSlotProps } from '@uifabricshared/theming-ramp';
import { processTokens, buildComponentTokens } from '@uifabricshared/foundation-tokens';
import { getTheme, ThemeContext } from '@uifabricshared/theming-react-native';
import { getMemoCache } from '@fluentui-react-native/memo-cache';
/* tslint:disable-next-line no-any */
export function getOptionsFromObj(obj) {
    var objType = obj && typeof obj;
    return ((objType === 'object' || objType === 'function') && obj.__composable) || undefined;
}
function _getSettingsFromTheme(theme, name) {
    return getSettings(theme, name);
}
function _getHasToken(slots) {
    var slotTokens = {};
    Object.keys(slots).forEach(function (slotName) {
        var slot = slots[slotName];
        var slotType = (typeof slot !== 'object' ? slot : slot.slotType);
        var options = getOptionsFromObj(slotType);
        slotTokens[slotName] = (options && options.resolvedTokens && options.resolvedTokens.tokenKeys) || undefined;
    });
    return function (target, key) {
        return slotTokens[target] && slotTokens[target].hasOwnProperty(key);
    };
}
function useStylingCore(props, options, instanceMemoCache, lookupOverride) {
    // get the theme value from the context (or the default theme if it is not set)
    var theme = React.useContext(ThemeContext) || getTheme();
    // resolve the array of settings for these options
    lookupOverride = lookupOverride || props;
    var _a = getThemedSettings(options.settings, theme, instanceMemoCache, lookupOverride, _getSettingsFromTheme), settings = _a.settings, getMemoValue = _a.getMemoValue;
    // finish by processing the tokens and turning IComponentSettings into ISlotProps (this removes things like _overrides)
    return returnAsSlotProps(processTokens(props, theme, settings, options.resolvedTokens, getMemoValue));
}
/**
 * return a useStyling implementation, in the form of IUseComposeStyling, based on the passed in styleSettings.  The
 * styleSettings will be captured in the created closure and will be set up to enable the appropriate levels of caching.
 *
 * @param options - style settings to configure this function.  Note that this should be scoped to a single component.
 * @param name - optional base name to use as a cache key
 */
export function initializeStyling(options) {
    // process the tokens and get them ready to render
    var styles = options.styles, slots = options.slots;
    options.resolvedTokens = buildComponentTokens(styles, _getHasToken(slots));
    // memo cache root for this component, keyed on options
    var getMemoValue = getMemoCache(options);
    // create a useStyling implementation for this component type (per type, not per instance)
    return function (props, lookupOverride) {
        return useStylingCore(props, options, getMemoValue, lookupOverride);
    };
}
//# sourceMappingURL=useStyling.js.map